@@ -1,6 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, unicode_literals
-
 """
 oauthlib.common
 ~~~~~~~~~~~~~~
@@ -8,13 +6,16 @@ oauthlib.common
 This module provides data structures and utilities common
 to all implementations of OAuth.
 """
+from __future__ import absolute_import, unicode_literals
 
 import collections
+import datetime
 import logging
 import random
 import re
 import sys
 import time
+
 try:
     from urllib import quote as _quote
     from urllib import unquote as _unquote
@@ -118,7 +119,7 @@ def decode_params_utf8(params):
     return decoded
 
 
-urlencoded = set(always_safe) | set('=&;%+~,')
+urlencoded = set(always_safe) | set('=&;%+~,*')
 
 
 def urldecode(query):
@@ -132,7 +133,12 @@ def urldecode(query):
     """
     # Check if query contains invalid characters
     if query and not set(query) <= urlencoded:
-        raise ValueError('Not a valid urlencoded string.')
+        error = ("Error trying to decode a non urlencoded string. "
+                 "Found invalid characters: %s "
+                 "in the string: '%s'. "
+                 "Please ensure the request/response body is "
+                 "x-www-form-urlencoded.")
+        raise ValueError(error % (set(query) - urlencoded, query))
 
     # Check for correctly hex encoded values using a regular expression
     # All encoded values begin with % followed by two hex characters
@@ -233,6 +239,40 @@ def generate_token(length=30, chars=UNICODE_ASCII_CHARACTER_SET):
     return ''.join(rand.choice(chars) for x in range(length))
 
 
+def generate_signed_token(private_pem, request):
+    import Crypto.PublicKey.RSA as RSA
+    import jwt
+
+    private_key = RSA.importKey(private_pem)
+
+    now = datetime.datetime.utcnow()
+
+    claims = {
+        'scope': request.scope,
+        'exp': now + datetime.timedelta(seconds=request.expires_in)
+    }
+
+    claims.update(request.claims)
+
+    token = jwt.encode(claims, private_key, 'RS256')
+    token = to_unicode(token, "UTF-8")
+
+    return token
+
+
+def verify_signed_token(private_pem, token):
+    import Crypto.PublicKey.RSA as RSA
+    import jwt
+
+    public_key = RSA.importKey(private_pem).publickey()
+
+    try:
+        #return jwt.verify_jwt(token.encode(), public_key)
+        return jwt.decode(token, public_key)
+    except:
+        raise Exception
+
+
 def generate_client_id(length=30, chars=CLIENT_ID_CHARACTER_SET):
     """Generates an OAuth client_id
 
@@ -244,10 +284,9 @@ def generate_client_id(length=30, chars=CLIENT_ID_CHARACTER_SET):
 
 def add_params_to_qs(query, params):
     """Extend a query with a list of two-tuples."""
-    if isinstance(query, dict):
-        queryparams = query.items()
-    else:
-        queryparams = urlparse.parse_qsl(query, keep_blank_values=True)
+    if isinstance(params, dict):
+        params = params.items()
+    queryparams = urlparse.parse_qsl(query, keep_blank_values=True)
     queryparams.extend(params)
     return urlencode(queryparams)
 
@@ -280,7 +319,7 @@ def safe_string_equals(a, b):
     return result == 0
 
 
-def to_unicode(data, encoding):
+def to_unicode(data, encoding='UTF-8'):
     """Convert a number of different types of objects to unicode."""
     if isinstance(data, unicode_type):
         return data
@@ -383,7 +422,7 @@ class Request(object):
     @property
     def duplicate_params(self):
         seen_keys = collections.defaultdict(int)
-        all_keys = (p[0] for p in self.decoded_body or [] + self.uri_query_params)
+        all_keys = (p[0] for p in (self.decoded_body or []) + self.uri_query_params)
         for k in all_keys:
             seen_keys[k] += 1
         return [k for k, c in seen_keys.items() if c > 1]
